using System;
using System.Data;
using System.Collections.Generic;
using Microsoft.VisualStudio.TestTools.UnitTesting;

namespace NBearLite.Test
{
    [TestClass]
    public class DatabaseTest
    {
        private static Database db;

        #region Additional test attributes
        //
        // You can use the following additional attributes as you write your tests:
        //
        // Use ClassInitialize to run code before running the first test in the class
        [ClassInitialize()]
        public static void MyClassInitialize(TestContext testContext)
        {
            if (db == null)
            {
                db = new Database("Northwind");
                db.OnLog += Console.WriteLine;
            }
        }
        //
        // Use ClassCleanup to run code after all tests in a class have run
        // [ClassCleanup()]
        // public static void MyClassCleanup() { }
        //
        // Use TestInitialize to run code before running each test 
        // [TestInitialize()]
        // public void MyTestInitialize() { }
        //
        // Use TestCleanup to run code after each test has run
        // [TestCleanup()]
        // public void MyTestCleanup() { }
        //
        #endregion

        [TestMethod]
        public void TestInsert()
        {
            Assert.AreEqual(1, db.Insert(Northwind.Categories).AddColumn(Northwind.Categories.CategoryName, "test1").Execute());
            Assert.IsTrue(db.Insert(Northwind.Categories).AddColumn(Northwind.Categories.CategoryName, "test1").ExecuteReturnAutoIncrementID(Northwind.Categories.CategoryID) > 1);
        }

        [TestMethod]
        public void TestUpdate()
        {
            Assert.AreEqual(1, db.Insert(Northwind.Categories).AddColumn(Northwind.Categories.CategoryName, "test1").Execute());
            Assert.IsTrue(db.Update(Northwind.Categories).AddColumn(Northwind.Categories.CategoryName, "test1").Where(Northwind.Categories.CategoryName == "test1").Execute() > 0);
        }

        [TestMethod]
        public void TestDelete()
        {
            Assert.AreEqual(1, db.Insert(Northwind.Categories).AddColumn(Northwind.Categories.CategoryName, "test111").Execute());
            Assert.IsTrue(db.Delete(Northwind.Categories).Where(Northwind.Categories.CategoryName == "test111").Execute() > 0);
        }

        [TestMethod]
        public void TestSelect()
        {
            DataSet ds = db.Select(Northwind.Categories).ToDataSet();
            Assert.IsTrue(ds.Tables[0].Rows.Count > 0);
            ds = db.Select(Northwind.Categories, Northwind.Categories.CategoryName).Where(Northwind.Categories.CategoryID > 0).ToDataSet();
            Assert.IsTrue(ds.Tables[0].Rows.Count > 0);
            ds = db.Select(Northwind.Categories, Northwind.Categories.CategoryName).GroupBy(Northwind.Categories.CategoryName).OrderBy(Northwind.Categories.CategoryName.Desc).SetSelectRange(2, 2, Northwind.Categories.CategoryName).ToDataSet();

            ds = db.Select(Northwind.Categories, QueryColumn.All(Northwind.Categories), QueryColumn.All(Northwind.Products))
                .Join(Northwind.Products, Northwind.Products.CategoryID == Northwind.Categories.CategoryID)
                .Where(Northwind.Categories.CategoryName.Length > 3 && Northwind.Products.UnitPrice > 100)
                .OrderBy(Northwind.Categories.CategoryID.Asc, Northwind.Products.ProductID.Desc)
                .ToDataSet();

            ds = db.Select(Northwind.Categories, Northwind.Categories.__Alias("CategoriesAlias").CategoryName).Join(Northwind.Categories, "CategoriesAlias", Northwind.Categories.CategoryID == Northwind.Categories.__Alias("CategoriesAlias").CategoryID).
                SetSelectRange(2, 2, Northwind.Categories.CategoryID).Where(Northwind.Categories.CategoryName.Length > 0 && Northwind.Categories.__Alias("CategoriesAlias").Description != null).
                ToDataSet();

            int pageSize = 10;
            int rowCount = (int)db.Select(Northwind.Categories, QueryColumn.All().Count()).ToScalar();
            int pageCount = (rowCount % 10 == 0 ? rowCount / 10 : (rowCount / 10) + 1);

            Console.WriteLine(string.Format("pageSize={0}, rowCount={1}, pageCount={2}", pageSize, rowCount, pageCount));
        }

        [TestMethod]
        public void TestPostgreSql()
        {
            Database pgDb = new Database("Postgres");
            pgDb.OnLog += Console.WriteLine;
            Assert.AreEqual(1, pgDb.Insert(Postgres.TestTable).AddColumn(Postgres.TestTable.Name, "test1").Execute());
            Assert.IsTrue(pgDb.Insert(Postgres.TestTable).AddColumn(Postgres.TestTable.Name, "test111").ExecuteReturnAutoIncrementID(Postgres.TestTable.ID) > 1);
            Assert.IsTrue(pgDb.Update(Postgres.TestTable).AddColumn(Postgres.TestTable.Name, "test111").Where(Postgres.TestTable.Name == "test111").Execute() > 0);
            Assert.IsTrue(pgDb.Delete(Postgres.TestTable).Where(Postgres.TestTable.Name == "test1").Execute() > 0);
            DataSet ds = pgDb.Select(Postgres.TestTable).ToDataSet();
            Assert.IsTrue(ds.Tables[0].Rows.Count > 0);
            ds = pgDb.Select(Postgres.TestTable).Where(Postgres.TestTable.ID > 0).ToDataSet();
            Assert.IsTrue(ds.Tables[0].Rows.Count > 0);
            ds = pgDb.Select(Postgres.TestTable, Postgres.TestTable.Name).GroupBy(Postgres.TestTable.Name).OrderBy(Postgres.TestTable.Name.Desc).SetSelectRange(2, 2, Postgres.TestTable.Name).ToDataSet();
            ds = pgDb.Select(Postgres.TestTable, Postgres.TestTable.__Alias("Alias").Name).Join(Postgres.TestTable, "Alias", Postgres.TestTable.ID == Postgres.TestTable.__Alias("Alias").ID).
                SetSelectRange(2, 2, Postgres.TestTable.ID).Where(Postgres.TestTable.Name.Length > 0).
                ToDataSet();

            //test postgresql datat ypes
            string shortText = "short";
            string longText = string.Empty;
            for (int i = 0; i < 10000; ++i)
            {
                longText += "t";
            }
            Assert.AreEqual(1, pgDb.Insert(Postgres.TestDataTypes).AddColumn(Postgres.TestDataTypes.VarCharColumn, shortText)
                .AddColumn(Postgres.TestDataTypes.TextColumn, longText).AddColumn(Postgres.TestDataTypes.DateTimeColumn, DateTime.Now).Execute());

            Assert.IsTrue(pgDb.Update(Postgres.TestDataTypes).AddColumn(Postgres.TestDataTypes.DateTimeColumn, DateTime.Now)
                .Where(Postgres.TestDataTypes.DateTimeColumn + new TimeSpan(1, 1, 1 ,1) > DateTime.Now).Execute() > 0);
        }

        [TestMethod]
        public void TestStoredProcedure()
        {
            int ret = -1;
            DataSet ds = Northwind.SalesbyYear(db, out ret, new DateTime(1800, 9, 9), DateTime.Now);
            Assert.IsTrue(ret >= 0);
            Console.WriteLine(ret.ToString());

            string outStr = "1";
            ds = Northwind.SalesByCategoryTest(db, out ret, "test1", "1997", ref outStr);
            Assert.IsTrue(ret >= 0);
            Assert.AreNotEqual("1", outStr);
        }

        [TestMethod]
        public void TestSubQuery()
        {
            DataSet ds = db.Select(Northwind.Products)
                .Where(Northwind.Products.CategoryID.In
                (
                    db.Select(Northwind.Categories, Northwind.Categories.CategoryID).SetSelectRange(10, 0, Northwind.Categories.CategoryID).ToSubQuery())
                )
                .ToDataSet();
            Assert.IsTrue(ds.Tables[0].Rows.Count > 0);

            ds = db.Select(Northwind.Products)
                .Where(Northwind.Products.CategoryID ==
                (
                    db.Select(Northwind.Categories, Northwind.Categories.CategoryID).SetSelectRange(1, 0, Northwind.Categories.CategoryID).ToSubQuery())
                )
                .ToDataSet();
            Assert.IsTrue(ds.Tables[0].Rows.Count > 0);
        }

        [TestMethod]
        public void TestSaveDataRowDataTable()
        {
            int catID = db.Insert(Northwind.Categories).AddColumn(Northwind.Categories.CategoryName, "test1").ExecuteReturnAutoIncrementID(Northwind.Categories.CategoryID);
            SelectSqlSection selectSection = db.Select(Northwind.Categories)
                .Where(Northwind.Categories.CategoryID == catID)
                .OrderBy(Northwind.Categories.CategoryID.Desc)
                .SetSelectRange(1, 0, Northwind.Categories.CategoryID);
            DataTable dt = selectSection.ToDataSet().Tables[0];

            Assert.IsTrue(dt.Rows.Count == 1);

            dt.Rows[0]["CategoryName"] = "modified";

            Assert.AreEqual(1, db.Save(selectSection.ToDbCommand(), 10, dt));

            Assert.AreEqual("modified", db.Select(Northwind.Categories, Northwind.Categories.CategoryName).Where(Northwind.Categories.CategoryID == catID).ToScalar().ToString());

            dt.Rows[0]["CategoryName"] = "modified2";

            DataRow newRow = dt.NewRow();
            newRow["CategoryName"] = "new";
            dt.Rows.Add(newRow);

            System.Data.Common.DbTransaction tran = db.BeginTransaction();
            Assert.AreEqual(2, db.Save(selectSection.ToDbCommand(), tran, 0, dt.Rows[0], dt.Rows[1]));
            tran.Commit();
            db.CloseConnection(tran);

            Assert.AreEqual("modified2", db.Select(Northwind.Categories, Northwind.Categories.CategoryName).Where(Northwind.Categories.CategoryID == catID).ToScalar().ToString());
            Assert.IsTrue(db.Select(Northwind.Categories, Northwind.Categories.CategoryName).SetSelectRange(1, 0, Northwind.Categories.CategoryID).OrderBy(Northwind.Categories.CategoryID.Desc).ToScalar().ToString() == "new");
        }

        [TestMethod]
        public void TestTransaction()
        {
            //ADO.NET 1.1 style transaction
            System.Data.Common.DbTransaction tran = db.BeginTransaction();
            db.Insert(Northwind.Categories).AddColumn(Northwind.Categories.CategoryName, "test1").SetTransaction(tran).Execute();
            tran.Commit();
            db.CloseConnection(tran);

            //ADO.NET 2.0 style transaction
            using (System.Transactions.TransactionScope scope = new System.Transactions.TransactionScope(System.Transactions.TransactionScopeOption.Required))
            {
                db.Insert(Northwind.Categories).AddColumn(Northwind.Categories.CategoryName, "test1").Execute();
                scope.Complete();
            }
        }

        [TestMethod]
        public void TestSqlServerBatchDatabase()
        {
            Database batchDB = new Database("Northwind");
            batchDB.OnLog += Console.WriteLine;
            batchDB.BeginBatchConnection(10);
            batchDB.Insert(Northwind.Categories).AddColumn(Northwind.Categories.CategoryName, "test1111").Execute();
            batchDB.Delete(Northwind.Categories).Where(Northwind.Categories.CategoryName == "test1111").Execute();
            batchDB.EndBatchConnection();
        }
    }
}
